<?php
//============================================================+
// File name   : tce_xml_questions.php
// Begin       : 2006-03-06
// Last Update : 2008-12-04
// 
// Description : Functions to export questions using XML
//               format.
//
// Author: Nicola Asuni
//
// (c) Copyright:
//               Nicola Asuni
//               Tecnick.com S.r.l.
//               Via della Pace, 11
//               09044 Quartucciu (CA)
//               ITALY
//               www.tecnick.com
//               info@tecnick.com
//
// License: 
//    Copyright (C) 2004-2008  Nicola Asuni - Tecnick.com S.r.l.
//    
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//    
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//    
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//     
//    Additionally, you can't remove the original TCExam logo, copyrights statements
//    and links to Tecnick.com and TCExam websites.
//    
//    See LICENSE.TXT file for more information.
//============================================================+

/**
 * Display all questions grouped by topic in XML format.
 * @package com.tecnick.tcexam.admin
 * @author Nicola Asuni
 * @copyright Copyright &copy; 2004-2008, Nicola Asuni - Tecnick.com S.r.l. - ITALY - www.tecnick.com - info@tecnick.com
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
 * @link www.tecnick.com
 * @since 2006-03-11
 */

/**
 */

// send XML headers
header('Content-Description: XML File Transfer');
header('Cache-Control: public, must-revalidate, max-age=0'); // HTTP/1.1
header('Pragma: public');
header('Expires: Sat, 26 Jul 1997 05:00:00 GMT'); // Date in the past
header('Last-Modified: '.gmdate('D, d M Y H:i:s').' GMT');
// force download dialog
header('Content-Type: application/force-download');
header('Content-Type: application/octet-stream', false);
header('Content-Type: application/download', false);
header('Content-Type: application/xml', false);
// use the Content-Disposition header to supply a recommended filename
header('Content-Disposition: attachment; filename=tcexam_questions_'.$_REQUEST['subject_id'].'_'.date('YmdHis').'.xml;');
header('Content-Transfer-Encoding: binary');

if ((isset($_REQUEST['expmode']) AND ($_REQUEST['expmode'] > 0))
	AND (isset($_REQUEST['module_id']) AND ($_REQUEST['module_id'] > 0))
	AND (isset($_REQUEST['subject_id']) AND ($_REQUEST['subject_id'] > 0))) {
	$expmode = intval($_REQUEST['expmode']);
	$module_id = intval($_REQUEST['module_id']);
	$subject_id = intval($_REQUEST['subject_id']);
	echo F_xml_export_questions($module_id, $subject_id, $expmode);
} else {
	exit;
}

/**
 * Export all questions of the selected subject to XML.
 * @author Nicola Asuni
 * @copyright Copyright &copy; 2004-2008, Nicola Asuni - Tecnick.com S.r.l. - ITALY - www.tecnick.com - info@tecnick.com
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
 * @link www.tecnick.com
 * @since 2006-03-06
 * @param int $module_id  module ID
 * @param int $subject_id topic ID
 * @param int $expmode export mode: 1 = selected topic; 2 = selected module; 3 = all modules.
 * @return XML data
 */
function F_xml_export_questions($module_id, $subject_id, $expmode) {
	global $l, $db;
	require_once('../config/tce_config.php');
	require_once('../../shared/code/tce_authorization.php');
	require_once('../code/tce_functions_auth_sql.php');
	$module_id = intval($module_id);
	$subject_id = intval($subject_id);
	$expmode = intval($expmode);
	
	$boolean = array('false', 'true');
	$type = array('single', 'multiple', 'text', 'ordering');

	$xml = ''; // XML data to be returned

	$xml .= "<"."?xml version=\"1.0\" encoding=\"UTF-8\" ?".">\n";
	$xml .= "<tcexamquestions version=\"".K_TCEXAM_VERSION."\">\n";
	$xml .= "\t<header";
	$xml .= " lang=\"".K_USER_LANG."\"";
	$xml .= " date=\"".date(K_TIMESTAMP_FORMAT)."\">\n";
	$xml .= "\t</header>\n";
	$xml .= "\t<body>\n";
	
	// ---- module
	$sqlm = 'SELECT * FROM '.K_TABLE_MODULES.'';
	if ($expmode < 3) {
		$sqlm .= ' WHERE module_id='.$module_id.'';
	}
	$sqlm .= ' ORDER BY module_name';
	if($rm = F_db_query($sqlm, $db)) {
		while($mm = F_db_fetch_array($rm)) {
			$xml .= "\t\t<module>\n";
			
			$xml .= "\t\t\t<name>";
			$xml .=  F_text_to_xml($mm['module_name']);
			$xml .= "</name>\n";
			
			$xml .= "\t\t\t<enabled>";
			$xml .= $boolean[intval(F_getBoolean($mm['module_enabled']))];
			$xml .= "</enabled>\n";
			
			// ---- topic
			$where_sqls = 'subject_module_id='.$mm['module_id'].'';
			if ($expmode < 2) {
				$where_sqls .= ' AND subject_id='.$subject_id.'';
			}
			$sqls = F_select_subjects_sql($where_sqls);
			if($rs = F_db_query($sqls, $db)) {
				while($ms = F_db_fetch_array($rs)) {
					$xml .= "\t\t\t<subject>\n";
					
					$xml .= "\t\t\t\t<name>";
					$xml .=  F_text_to_xml($ms['subject_name']);
					$xml .= "</name>\n";
					
					$xml .= "\t\t\t\t<description>";
					$xml .=  F_text_to_xml($ms['subject_description']);
					$xml .= "</description>\n";
								
					$xml .= "\t\t\t\t<enabled>";
					$xml .= $boolean[intval(F_getBoolean($ms['subject_enabled']))];
					$xml .= "</enabled>\n";
					
					// ---- questions
					$sql = "SELECT * 
						FROM ".K_TABLE_QUESTIONS."
						WHERE question_subject_id=".$ms['subject_id']."
						ORDER BY question_enabled DESC, question_position, question_description";
					if($r = F_db_query($sql, $db)) {
						while($m = F_db_fetch_array($r)) {
			
							$xml .= "\t\t\t\t<question>\n";
			
							$xml .= "\t\t\t\t\t<enabled>";
							$xml .= $boolean[intval(F_getBoolean($m['question_enabled']))];
							$xml .= "</enabled>\n";
			
							$xml .= "\t\t\t\t\t<type>";
							$xml .= $type[$m['question_type']-1];
							$xml .= "</type>\n";
			
							$xml .= "\t\t\t\t\t<difficulty>";
							$xml .= $m['question_difficulty'];
							$xml .= "</difficulty>\n";
			
							$xml .= "\t\t\t\t\t<position>";
							$xml .= $m['question_position'];
							$xml .= "</position>\n";
			
							$xml .= "\t\t\t\t\t<timer>";
							$xml .= $m['question_timer'];
							$xml .= "</timer>\n";
			
							$xml .= "\t\t\t\t\t<fullscreen>";
							$xml .= $boolean[intval(F_getBoolean($m['question_fullscreen']))];
							$xml .= "</fullscreen>\n";
			
							$xml .= "\t\t\t\t\t<inline_answers>";
							$xml .= $boolean[intval(F_getBoolean($m['question_inline_answers']))];
							$xml .= "</inline_answers>\n";
			
							$xml .= "\t\t\t\t\t<auto_next>";
							$xml .= $boolean[intval(F_getBoolean($m['question_auto_next']))];
							$xml .= "</auto_next>\n";
			
							$xml .= "\t\t\t\t\t<description>";
							$xml .=  F_text_to_xml($m['question_description']);
							$xml .= "</description>\n";
			
							// display alternative answers
							$sqla = "SELECT *
								FROM ".K_TABLE_ANSWERS."
								WHERE answer_question_id='".$m['question_id']."'
								ORDER BY answer_position,answer_isright DESC";
							if($ra = F_db_query($sqla, $db)) {
								while($ma = F_db_fetch_array($ra)) {
					
									$xml .= "\t\t\t\t\t<answer>\n";
					
									$xml .= "\t\t\t\t\t\t<enabled>";
									$xml .= $boolean[intval(F_getBoolean($ma['answer_enabled']))];
									$xml .= "</enabled>\n";
					
									$xml .= "\t\t\t\t\t\t<isright>";
									$xml .= $boolean[intval(F_getBoolean($ma['answer_isright']))];
									$xml .= "</isright>\n";
					
									$xml .= "\t\t\t\t\t\t<position>";
									$xml .= $ma['answer_position'];
									$xml .= "</position>\n";
					
									$xml .= "\t\t\t\t\t\t<key>";
									$xml .= F_text_to_xml(chr($ma['answer_keyboard_key']));
									$xml .= "</key>\n";
					
									$xml .= "\t\t\t\t\t\t<description>";
									$xml .=  F_text_to_xml($ma['answer_description']);
									$xml .= "</description>\n";
					
									$xml .= "\t\t\t\t\t</answer>\n";
								}
							} else {
								F_display_db_error();
							}
							
							$xml .= "\t\t\t\t</question>\n";
							
						} // end while for questions
					} else {
						F_display_db_error();
					}
					
					$xml .= "\t\t\t</subject>\n";
					
				} // end while for topics
			} else {
				F_display_db_error();
			}
			
			$xml .= "\t\t</module>\n";
			
		} // end while for module
	} else {
		F_display_db_error();
	}
	
	$xml .= "\t</body>\n";
	$xml .= "</tcexamquestions>\n";
	
	return $xml;
}

//============================================================+
// END OF FILE                                                 
//============================================================+
?>
