<?php
//============================================================+
// File name   : tce_xml_results.php
// Begin       : 2008-06-06
// Last Update : 2008-11-29
// 
// Description : Export all users' results in XML.
// 
// Author: Nicola Asuni
//
// (c) Copyright:
//               Nicola Asuni
//               Tecnick.com S.r.l.
//               Via della Pace, 11
//               09044 Quartucciu (CA)
//               ITALY
//               www.tecnick.com
//               info@tecnick.com
//
// License: 
//    Copyright (C) 2004-2008  Nicola Asuni - Tecnick.com S.r.l.
//    
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//    
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//    
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//     
//    Additionally, you can't remove the original TCExam logo, copyrights statements
//    and links to Tecnick.com and TCExam websites.
//    
//    See LICENSE.TXT file for more information.
//============================================================+

/**
 * Export all users' results in XML.
 * @package com.tecnick.tcexam.admin
 * @author Nicola Asuni
 * @copyright Copyright &copy; 2004-2008, Nicola Asuni - Tecnick.com S.r.l. - ITALY - www.tecnick.com - info@tecnick.com
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
 * @link www.tecnick.com
 * @since 2004-06-11
 * @param int $_REQUEST['testid'] test ID
 * @param int $_REQUEST['groupid'] group ID
 * @param string $_REQUEST['orderfield'] ORDER BY portion of SQL selection query
 */

/**
 */

require_once('../config/tce_config.php');
require_once('../../shared/code/tce_functions_tcecode.php');
require_once('../../shared/code/tce_functions_test.php');
require_once('../../shared/code/tce_functions_test_stats.php');

if (isset($_REQUEST['testid']) AND ($_REQUEST['testid'] > 0)) {
	$test_id = intval($_REQUEST['testid']);
	// check user's authorization
	require_once('../../shared/code/tce_authorization.php');
	if (!F_isAuthorizedUser(K_TABLE_TESTS, 'test_id', $test_id, 'test_user_id')) {
		exit;
	}
} else {
	exit;
}

if (isset($_REQUEST['groupid']) AND ($_REQUEST['groupid'] > 0)) {
	$group_id = intval($_REQUEST['groupid']);
} else {
	$group_id = 0;
}

// define symbols for answers list
$qtype = array('S', 'M', 'T', 'O'); // question types
$type = array('single', 'multiple', 'text', 'ordering');
$boolean = array('false', 'true');

// send XML headers
header('Content-Description: XML File Transfer');
header('Cache-Control: public, must-revalidate, max-age=0'); // HTTP/1.1
header('Pragma: public');
header('Expires: Sat, 26 Jul 1997 05:00:00 GMT'); // Date in the past
header('Last-Modified: '.gmdate('D, d M Y H:i:s').' GMT');
// force download dialog
header('Content-Type: application/force-download');
header('Content-Type: application/octet-stream', false);
header('Content-Type: application/download', false);
header('Content-Type: application/xml', false);
// use the Content-Disposition header to supply a recommended filename
header('Content-Disposition: attachment; filename=tcexam_results_'.$_REQUEST['testid'].'_'.date('YmdHis').'.xml;');
header('Content-Transfer-Encoding: binary');

$xml = ''; // XML data to be returned

$xml .= "<"."?xml version=\"1.0\" encoding=\"UTF-8\" ?".">\n";
$xml .= "<tcexamresults version=\"".K_TCEXAM_VERSION."\">\n";
$xml .= "\t<header";
$xml .= " lang=\"".K_USER_LANG."\"";
$xml .= " date=\"".date(K_TIMESTAMP_FORMAT)."\">\n";
$xml .= "\t</header>\n";
$xml .= "\t<body>\n";

// get test data
$sql = "SELECT * 
	FROM ".K_TABLE_TESTS." 
	WHERE test_id=".$test_id."
	LIMIT 1";
if($r = F_db_query($sql, $db)) {
	if($m = F_db_fetch_array($r)) {
		// test data
		$xml .= "\t\t<test id=\"".$test_id."\">\n";
		$xml .= "\t\t\t<name>".F_text_to_xml($m['test_name'])."</name>\n";
		$xml .= "\t\t\t<description>".F_text_to_xml($m['test_description'])."</description>\n";
		$xml .= "\t\t\t<begin_time>".$m['test_begin_time']."</begin_time>\n";
		$xml .= "\t\t\t<end_time>".$m['test_end_time']."</end_time>\n";
		$xml .= "\t\t\t<duration_time>".$m['test_duration_time']."</duration_time>\n";
		$xml .= "\t\t\t<ip_range>".$m['test_ip_range']."</ip_range>\n";
		$xml .= "\t\t\t<score_right>".$m['test_score_right']."</score_right>\n";
		$xml .= "\t\t\t<score_wrong>".$m['test_score_wrong']."</score_wrong>\n";
		$xml .= "\t\t\t<score_unanswered>".$m['test_score_unanswered']."</score_unanswered>\n";
		$xml .= "\t\t\t<score_threshold>".$m['test_score_threshold']."</score_threshold>\n";
		$xml .= "\t\t\t<max_score>".$m['test_max_score']."</max_score>\n";
		$xml .= "\t\t\t<random_questions_select>".$boolean[intval(F_getBoolean($m['test_random_questions_select']))]."</random_questions_select>\n";
		$xml .= "\t\t\t<random_questions_order>".$boolean[intval(F_getBoolean($m['test_random_questions_order']))]."</random_questions_order>\n";
		$xml .= "\t\t\t<random_answers_select>".$boolean[intval(F_getBoolean($m['test_random_answers_select']))]."</random_answers_select>\n";
		$xml .= "\t\t\t<random_answers_order>".$boolean[intval(F_getBoolean($m['test_random_answers_order']))]."</random_answers_order>\n";
		$xml .= "\t\t\t<results_to_users>".$boolean[intval(F_getBoolean($m['test_results_to_users']))]."</results_to_users>\n";
		$xml .= "\t\t\t<report_to_users>".$boolean[intval(F_getBoolean($m['test_report_to_users']))]."</report_to_users>\n";
		$xml .= "\t\t\t<comment_enabled>".$boolean[intval(F_getBoolean($m['test_comment_enabled']))]."</comment_enabled>\n";
		$xml .= "\t\t\t<menu_enabled>".$boolean[intval(F_getBoolean($m['test_menu_enabled']))]."</menu_enabled>\n";
		$xml .= "\t\t\t<noanswer_enabled>".$boolean[intval(F_getBoolean($m['test_noanswer_enabled']))]."</noanswer_enabled>\n";
		$xml .= "\t\t\t<mcma_radio>".$boolean[intval(F_getBoolean($m['test_mcma_radio']))]."</mcma_radio>\n";
		$xml .= "\t\t</test>\n";
	}
} else {
	F_display_db_error();
}

$sql = "SELECT testuser_id, user_id, SUM(testlog_score) AS total_score, MAX(testlog_change_time) AS test_end_time, testuser_creation_time, testuser_comment
	FROM ".K_TABLE_TESTS_LOGS.", ".K_TABLE_TEST_USER.", ".K_TABLE_USERS." 
	WHERE testlog_testuser_id=testuser_id
		AND testuser_user_id=user_id 
		AND testuser_test_id=".$test_id."";
if ($group_id > 0) {
	$sql .= " AND testuser_user_id IN (
			SELECT usrgrp_user_id
			FROM ".K_TABLE_USERGROUP." 
			WHERE usrgrp_group_id=".$group_id."
		)";
}
$sql .= " GROUP BY testuser_id, user_id, testuser_creation_time, testuser_comment";
if($r = F_db_query($sql, $db)) {
	while($m = F_db_fetch_array($r)) {
		$testuser_id = $m['testuser_id'];
		$user_id = $m['user_id'];
		$xml .= "\t\t<user id=\"".$user_id."\">\n";			
		$sqla = "SELECT * 
		FROM ".K_TABLE_USERS."
		WHERE user_id=".$user_id."
		LIMIT 1";
		if($ra = F_db_query($sqla, $db)) {
			$xml .= "\t\t\t<userdata>\n";
			if($ma = F_db_fetch_array($ra)) {		
				$xml .= "\t\t\t\t<name>".F_text_to_xml($ma['user_name'])."</name>\n";		
				$xml .= "\t\t\t\t<email>".$ma['user_email']."</email>\n";				
				$xml .= "\t\t\t\t<regdate>".$ma['user_regdate']."</regdate>\n";				
				$xml .= "\t\t\t\t<ip>".$ma['user_ip']."</ip>\n";				
				$xml .= "\t\t\t\t<firstname>".F_text_to_xml($ma['user_firstname'])."</firstname>\n";				
				$xml .= "\t\t\t\t<lastname>".F_text_to_xml($ma['user_lastname'])."</lastname>\n";				
				$xml .= "\t\t\t\t<birthdate>".$ma['user_birthdate']."</birthdate>\n";				
				$xml .= "\t\t\t\t<birthplace>".F_text_to_xml($ma['user_birthplace'])."</birthplace>\n";
				$xml .= "\t\t\t\t<regnumber>".F_text_to_xml($ma['user_regnumber'])."</regnumber>\n";
				$xml .= "\t\t\t\t<ssn>".F_text_to_xml($ma['user_ssn'])."</ssn>\n";
				$xml .= "\t\t\t\t<level>".$ma['user_level']."</level>\n";
				$xml .= "\t\t\t\t<verifycode>".$ma['user_verifycode']."</verifycode>\n";
			}
			$xml .= "\t\t\t</userdata>\n";
		}
		$usrtestdata = F_getUserTestStat($test_id, $user_id);
		$xml .= "\t\t\t<stats>\n";
		$xml .= "\t\t\t\t<score>".round($m['total_score'],1)."</score>\n";
		$xml .= "\t\t\t\t<score_percent>".round(100 * $usrtestdata['score'] / $usrtestdata['max_score'])."</score_percent>\n";					
		$xml .= "\t\t\t\t<right>".$usrtestdata['right']."</right>\n";
		$xml .= "\t\t\t\t<right_percent>".round(100 * $usrtestdata['right'] / $usrtestdata['all'])."</right_percent>\n";						
		$xml .= "\t\t\t\t<wrong>".$usrtestdata['wrong']."</wrong>\n";
		$xml .= "\t\t\t\t<wrong_percent>".round(100 * $usrtestdata['wrong'] / $usrtestdata['all'])."</wrong_percent>\n";						
		$xml .= "\t\t\t\t<unanswered>".$usrtestdata['unanswered']."</unanswered>\n";
		$xml .= "\t\t\t\t<unanswered_percent>".round(100 * $usrtestdata['unanswered'] / $usrtestdata['all'])."</unanswered_percent>\n";				
		$xml .= "\t\t\t\t<undisplayed>".$usrtestdata['undisplayed']."</undisplayed>\n";
		$xml .= "\t\t\t\t<undisplayed_percent>".round(100 * $usrtestdata['undisplayed'] / $usrtestdata['all'])."</undisplayed_percent>\n";
		$xml .= "\t\t\t\t<start_time>".$m['testuser_creation_time']."</start_time>\n";
		$xml .= "\t\t\t\t<end_time>".$m['test_end_time']."</end_time>\n";
		$xml .= "\t\t\t\t<comment>".F_text_to_xml($m['testuser_comment'])."</comment>\n";
		$xml .= "\t\t\t</stats>\n";
		
		$xml .= "\t\t\t<details>\n";
		// detailed test report
		$sqlq = "SELECT * 
			FROM ".K_TABLE_QUESTIONS.",".K_TABLE_TESTS_LOGS." 
			WHERE question_id=testlog_question_id 
			AND testlog_testuser_id=".$testuser_id."
			ORDER BY testlog_id";
		if($rq = F_db_query($sqlq, $db)) {
			$itemcount = 1;
			while($mq = F_db_fetch_array($rq)) {
				$xml .= "\t\t\t\t<question num=\"".$itemcount."\" id=\"".$mq['question_id']."\">\n";
				$xml .= "\t\t\t\t\t<subject_id>".$mq['question_subject_id']."</subject_id>\n";
				$xml .= "\t\t\t\t\t<type>".$qtype[($mq['question_type']-1)]."</type>\n";
				$xml .= "\t\t\t\t\t<difficulty>".$mq['question_difficulty']."</difficulty>\n";
				$xml .= "\t\t\t\t\t<position>".$mq['question_position']."</position>\n";
				$xml .= "\t\t\t\t\t<timer>".$mq['question_timer']."</timer>\n";
				$xml .= "\t\t\t\t\t<fullscreen>".$boolean[intval(F_getBoolean($mq['question_fullscreen']))]."</fullscreen>\n";
				$xml .= "\t\t\t\t\t<inline_answers>".$boolean[intval(F_getBoolean($mq['question_inline_answers']))]."</inline_answers>\n";
				$xml .= "\t\t\t\t\t<auto_next>".$boolean[intval(F_getBoolean($mq['question_auto_next']))]."</auto_next>\n";
				$xml .= "\t\t\t\t\t<score>".$mq['testlog_score']."</score>\n";
				$xml .= "\t\t\t\t\t<ip>".$mq['testlog_user_ip']."</ip>\n";
				if (isset($mq['testlog_display_time']) AND (strlen($mq['testlog_display_time']) > 0)) {
					$display_time =  substr($mq['testlog_display_time'], 11, 8);
				} else {
					$display_time =  "";
				}
				$xml .= "\t\t\t\t\t<display_time>".$display_time."</display_time>\n";
				if (isset($mq['testlog_change_time']) AND (strlen($mq['testlog_change_time']) > 0)) {
					$change_time = substr($mq['testlog_change_time'], 11, 8);
				} else {
					$change_time = '';
				}
				$xml .= "\t\t\t\t\t<change_time>".$change_time."</change_time>\n";
				if (isset($mq['testlog_display_time']) AND isset($mq['testlog_change_time'])) {
					$diff_time = date("i:s", (strtotime($mq['testlog_change_time']) - strtotime($mq['testlog_display_time'])));
				} else {
					$diff_time = '';
				}
				$xml .= "\t\t\t\t\t<diff_time>".$diff_time."</diff_time>\n";
				if (isset($mq['testlog_reaction_time']) AND ($mq['testlog_reaction_time'] > 0)) {
					$reaction_time =  intval($mq['testlog_reaction_time']);
				} else {
					$reaction_time =  "";
				}
				$xml .= "\t\t\t\t\t<reaction_time>".$reaction_time."</reaction_time>\n";
				$xml .= "\t\t\t\t\t<description>".F_text_to_xml($mq['question_description'])."</description>\n";
				$xml .= "\t\t\t\t\t<answers>\n";
				if ($mq['question_type'] == 3) {
					// free-text question
					// print user text answer
					$xml .= "\t\t\t\t\t\t<text>".$mq['testlog_answer_text']."</text>\n";
				} else {
					// display each answer option
					$sqla = "SELECT *
						FROM ".K_TABLE_LOG_ANSWER.", ".K_TABLE_ANSWERS."
						WHERE logansw_answer_id=answer_id
							AND logansw_testlog_id='".$mq['testlog_id']."'
						ORDER BY logansw_order";
					if($ra = F_db_query($sqla, $db)) {
						$idx = 1; // count answer items
						while($ma = F_db_fetch_array($ra)) {
							$xml .= "\t\t\t\t\t\t<answer num=\"".$idx."\" id=\"".$ma['answer_id']."\">\n";
							if ($mq['question_type'] == 4) {	
								$xml .= "\t\t\t\t\t\t\t<position>".$ma['answer_position']."</position>\n";
								$xml .= "\t\t\t\t\t\t\t<answposition>".$ma['logansw_position']."</answposition>\n";
							} else {
								$xml .= "\t\t\t\t\t\t\t<isright>".$boolean[intval(F_getBoolean($ma['answer_isright']))]."</isright>\n";
								$xml .= "\t\t\t\t\t\t\t<selected>".intval($ma['logansw_selected'])."</selected>\n";
							}
							$xml .= "\t\t\t\t\t\t\t<key>".F_text_to_xml(chr($ma['answer_keyboard_key']))."</key>\n";
							$xml .= "\t\t\t\t\t\t\t<description>".F_text_to_xml($ma['answer_description'])."</description>\n";
							$idx++;
							$xml .= "\t\t\t\t\t\t</answer>\n";
						}
					} else {
						F_display_db_error();
					}
				} // end multiple answers
				$xml .= "\t\t\t\t\t</answers>\n";
				// export teacher/supervisor comment to the question
				if (isset($mq['testlog_comment']) AND (!empty($mq['testlog_comment']))) {
					$xml .= "\t\t\t\t\t<qcomment>".F_text_to_xml($mq['testlog_comment'])."</qcomment>\n";
				}
				$xml .= "\t\t\t\t</question>\n";
				$itemcount++;
			} // end questions
			
			$xml .= "\t\t\t</details>\n";
		} else {
			F_display_db_error();
		}
	$xml .= "\t\t</user>\n";
	} 
} else {
	F_display_db_error();
}
$xml .= "\t</body>\n";
$xml .= "</tcexamresults>\n";
echo $xml;

//============================================================+
// END OF FILE                                                 
//============================================================+
?>
